from django.contrib import admin
from django.forms import Select
from django.urls import reverse
from django.utils.safestring import mark_safe

from .models import Manifest, Canvas


class ManifestSelectWidget(Select):
    '''Select widget for a :class:`~djiffy.models.Manifest` that also
    displays thumbnail and links to view and edit the Manifest.'''

    def render(self, name, value, attrs=None):
        widget = super(ManifestSelectWidget, self).render(name, value, attrs)
        if value:
            manifest = Manifest.objects.get(pk=value)
            # NOTE: should really only display edit link if user has
            # the change permission on manifests
            # FIXME: formatting / layout issues
            return mark_safe(
                '<div>%s</div><br/> <div style="float:left">%s <a href="%s">view</a> | <a href="%s">edit</a></div>' % \
                    (widget, manifest.admin_thumbnail() or '',
                     manifest.get_absolute_url(),
                     reverse('admin:djiffy_manifest_change', args=[manifest.id]))
                )

        return widget


class CanvasInline(admin.StackedInline):
    model = Canvas


class ManifestAdmin(admin.ModelAdmin):
    inlines = [CanvasInline]
    list_display = ('admin_thumbnail', 'label', 'short_id', 'created',
        'last_modified')
    list_display_links = ('admin_thumbnail', 'label')
    search_fields = ('label', 'short_id', 'uri', 'metadata', 'extra_data')


class CanvasAdmin(admin.ModelAdmin):
    list_display = ('admin_thumbnail', 'label', 'short_id', 'manifest')


admin.site.register(Manifest, ManifestAdmin)
admin.site.register(Canvas, CanvasAdmin)




'''
Generic manage command for importing IIIF Collections or manifests
into the database. Supports collections and individual manifests,
and local file paths as well as URLs.
'''

from django.core.management.base import BaseCommand

from djiffy.importer import ManifestImporter

class Command(BaseCommand):
    '''Import IIIF Collections or Manifests into the local database.'''
    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument('path', nargs='+',
            help='One or more IIIF Collections or Manifests as file or URL')
        parser.add_argument('--update', action='store_true',
            help='Update previously imported manifests')

    def handle(self, *args, **kwargs):
        ManifestImporter(stdout=self.stdout, stderr=self.stderr,
                         style=self.style, update=kwargs['update']) \
            .import_paths(kwargs['path'])






# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-10 17:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djiffy', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='canvas',
            options={'ordering': ['manifest', 'order'], 'permissions': (('view_manifest', 'Can view IIIF Canvas'),), 'verbose_name': 'IIIF Canvas', 'verbose_name_plural': 'IIIF Canvases'},
        ),
        migrations.AlterModelOptions(
            name='manifest',
            options={'permissions': (('view_canvas', 'Can view IIIF Manifest'),), 'verbose_name': 'IIIF Manifest'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-26 15:35
from __future__ import unicode_literals

import collections
from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djiffy', '0002_view_permissions'),
    ]

    operations = [
        migrations.AddField(
            model_name='canvas',
            name='extra_data',
            field=jsonfield.fields.JSONField(default=collections.OrderedDict),
        ),
        migrations.AlterField(
            model_name='manifest',
            name='extra_data',
            field=jsonfield.fields.JSONField(default=collections.OrderedDict),
        ),
    ]



import os
from setuptools import find_packages, setup
from djiffy import __version__

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

test_requirements = ['pytest>=5.1', 'pytest-django', 'pytest-cov',
                     'psycopg2-binary'],
# pytest v3.6 required for pytest-django but doesn't happen on travis-ci

setup(
    name='djiffy',
    version=__version__,
    packages=find_packages(),
    include_package_data=True,
    license='Apache License, Version 2.0',
    description='Django app for IIIF Presentation book content',
    long_description=README,
    url='https://github.com/Princeton-CDH/djiffy',
    install_requires=[
        'django>=3.0',
        'requests',
        'piffle',
        'attrdict',
        'jsonfield<3.0,>=2.1',
        'django-autocomplete-light>=3.5.1',
        'rdflib>6.0.1',
    ],
    setup_requires=['pytest-runner'],
    tests_require=test_requirements,
    extras_require={
        'test': test_requirements,
        'docs': ['sphinx']
    },
    author='CDH @ Princeton',
    author_email='cdhdevteam@princeton.edu',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



from dal import autocomplete
from django.db.models import Q
from django.http import Http404
from django.views.generic import DetailView, ListView

from djiffy.models import Canvas, Manifest


class ManifestList(ListView):
    '''List view for :class:`~djiffy.models.Manifest`.  Rendered with
    djiffy/manifest_list.html template.
    '''
    model = Manifest
    template_name = 'djiffy/manifest_list.html'
    context_object_name = 'manifests'


class ManifestDetail(DetailView):
    '''Detail view for a single :class:`~djiffy.models.Manifest`.
    Rendered with  djiffy/manifest_detail.html template.
    '''
    model = Manifest
    template_name = 'djiffy/manifest_detail.html'
    context_object_name = 'manifest'

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = Manifest.objects.all()

        try:
            return queryset.get(short_id=self.kwargs['id'])
        except queryset.model.DoesNotExist:
            raise Http404("No manifest found with id %(id)s" % self.kwargs)


class CanvasDetail(DetailView):
    '''Detail view for a single :class:`~djiffy.models.Canvas`.
    Rendered with  djiffy/canvast_detail.html template.
    '''
    model = Canvas
    template_name = 'djiffy/canvas_detail.html'
    context_object_name = 'canvas'

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = Canvas.objects.all()

        try:
            return queryset.get(short_id=self.kwargs['id'],
                manifest__short_id=self.kwargs['manifest_id'])
        except queryset.model.DoesNotExist:
            raise Http404("No canvas found with id %(id)s and manifest %(manifest_id)s" % \
                self.kwargs)


class CanvasAutocomplete(autocomplete.Select2QuerySetView):
    '''Canvas autocomplete view, e.g. for admin interface lookup'''
    def get_queryset(self):
        return Canvas.objects.filter(
            Q(label__icontains=self.q) |
            Q(uri__contains=self.q) |
            Q(manifest__label__icontains=self.q)
        )
